@@ -36,7 +36,8 @@ from zope.schema.fieldproperty import FieldProperty
 from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary
 
 from pyams_i18n.interfaces import II18n
-from pyams_security.interfaces import ILocalUser, ISecurityManager, IUsersFolderPlugin
+from pyams_security.interfaces import ILocalUser, ISecurityManager, IUsersFolderPlugin, \
+    PRINCIPAL_ID_FORMATTER, SALT_SIZE
 from pyams_security.interfaces.base import IPrincipalInfo
 from pyams_security.interfaces.names import USERS_FOLDERS_VOCABULARY_NAME
 from pyams_security.interfaces.notification import INotificationSettings
@@ -54,6 +55,7 @@ __docformat__ = 'restructuredtext'
 
 from pyams_security import _  # pylint: disable=ungrouped-imports
 
+
 LOGGER = logging.getLogger('PyAMS (security)')
 
 
@@ -79,8 +81,8 @@ class UsersFolder(Folder):
         if principal is not None:
             password = attrs.get('password')
             if principal.check_password(password):
-                return "{prefix}:{login}".format(prefix=self.prefix,
-                                                 login=principal.login)
+                return PRINCIPAL_ID_FORMATTER.format(prefix=self.prefix,
+                                                     login=principal.login)
         return None
 
     def check_login(self, login):
@@ -99,8 +101,8 @@ class UsersFolder(Folder):
         user = self.get(login)
         if user is not None:
             if info:
-                return PrincipalInfo(id='{prefix}:{login}'.format(prefix=self.prefix,
-                                                                  login=user.login),
+                return PrincipalInfo(id=PRINCIPAL_ID_FORMATTER.format(prefix=self.prefix,
+                                                                      login=user.login),
                                      title=user.title)
         return user
 
@@ -112,7 +114,7 @@ class UsersFolder(Folder):
             return {principal_id}
         return set()
 
-    def find_principals(self, query):
+    def find_principals(self, query, exact_match=False):
         """Get iterator of principals matching given query"""
         if not self.enabled:
             return
@@ -120,14 +122,20 @@ class UsersFolder(Folder):
             return
         query = query.lower()
         for user in self.values():
-            if (query == user.login or
-                    query in user.firstname.lower() or
-                    query in user.lastname.lower() or
-                    query in user.email.lower()):
-                yield PrincipalInfo(id='{prefix}:{login}'.format(prefix=self.prefix,
-                                                                 login=user.login),
-                                    title='{title} <{email}>'.format(title=user.title,
-                                                                     email=user.email))
+            if exact_match:
+                attrs = (user.login,)
+            else:
+                attrs = (user.login, user.firstname, user.lastname, user.email)
+            for attr in attrs:
+                if not attr:
+                    continue
+                if (exact_match and query == attr.lower()) or \
+                        (not exact_match and query in attr.lower()):
+                    yield PrincipalInfo(id=PRINCIPAL_ID_FORMATTER.format(prefix=self.prefix,
+                                                                         login=user.login),
+                                        title='{title} <{email}>'.format(title=user.title,
+                                                                         email=user.email))
+                    break
 
     def get_search_results(self, data):
         """Search iterator of principals matching given data"""
@@ -153,7 +161,7 @@ class UsersFolderVocabulary(SimpleVocabulary):
             for name, plugin in manager.items():
                 if IUsersFolderPlugin.providedBy(plugin):
                     terms.append(SimpleTerm(name, title=plugin.title))
-        super(UsersFolderVocabulary, self).__init__(terms)
+        super().__init__(terms)
 
 
 #
@@ -231,9 +239,9 @@ class LocalUser(Persistent, Contained):
     firstname = FieldProperty(ILocalUser['firstname'])
     lastname = FieldProperty(ILocalUser['lastname'])
     company_name = FieldProperty(ILocalUser['company_name'])
+    password_manager = FieldProperty(ILocalUser['password_manager'])
     _password = FieldProperty(ILocalUser['password'])
     _password_salt = None
-    password_manager = FieldProperty(ILocalUser['password_manager'])
     wait_confirmation = FieldProperty(ILocalUser['wait_confirmation'])
     self_registered = FieldProperty(ILocalUser['self_registered'])
     activation_secret = FieldProperty(ILocalUser['activation_secret'])
@@ -258,7 +266,7 @@ class LocalUser(Persistent, Contained):
         if value:
             if value == '*****':
                 return
-            self._password_salt = urandom(4)
+            self._password_salt = urandom(SALT_SIZE.get(self.password_manager, 4))
             manager = get_utility(IPasswordManager, name=self.password_manager)
             if self.password_manager == 'Plain Text':
                 self._password = manager.encodePassword(value)
@@ -331,8 +339,8 @@ class LocalUser(Persistent, Contained):
 @adapter_config(required=ILocalUser, provides=IPrincipalInfo)
 def user_principal_info_adapter(user):
     """User principal info adapter"""
-    return PrincipalInfo(id="{prefix}:{login}".format(prefix=user.__parent__.prefix,
-                                                      login=user.login),
+    return PrincipalInfo(id=PRINCIPAL_ID_FORMATTER.format(prefix=user.__parent__.prefix,
+                                                          login=user.login),
                          title=user.title)
 
 
